@@ -7,6 +7,7 @@ from random import SystemRandom
 import time
 from torch.utils.data import DataLoader, Dataset
 import pickle
+from copy import deepcopy
 
 from .data_utils import transform_seq, mtan_Dataset
 from .encoders import default_regressor, default_classifier, mtan_enc
@@ -301,7 +302,6 @@ class mtse:
         NotImplementedError, ValueError
         """
 
-        val_loss_fn = val_loss_fn if val_loss_fn is not None else lossf
 
         print('\n### Training ###')
         
@@ -319,6 +319,8 @@ class mtse:
         if CEweights is not None:
             criterion = nn.CrossEntropyLoss(torch.tensor(CEweights).to(self.device).float())
 
+        val_loss_fn = val_loss_fn if val_loss_fn is not None else criterion
+
         for itr in range(1, n_iters + 1):        
             train_loss = 0; train_n = 0
             start_time = time.time()
@@ -348,7 +350,7 @@ class mtse:
                 print('\t\t saving best model')
                 fname = str(self.experiment_id) + '_best.h5'
                 torch.save({'epoch': itr, 'model_state_dict': self.ts_encoder.state_dict(), 'optimizer_state_dict': self.optimizer.state_dict(), 'loss': -loss}, fname)
-                self.best_model = self.ts_encoder
+                self.best_model = deepcopy(self.ts_encoder)
 
             if self.early_stop > 0:
                 self.Early_stop(val_loss)
